@@ -24,9 +24,11 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <gthumb.h>
+#include "dlg-comments-preferences.h"
 #include "gth-comment.h"
 #include "gth-metadata-provider-comment.h"
 #include "gth-test-category.h"
+#include "preferences.h"
 
 
 GthMetadataCategory comments_metadata_category[] = {
@@ -36,12 +38,12 @@ GthMetadataCategory comments_metadata_category[] = {
 
 
 GthMetadataInfo comments_metadata_info[] = {
-	{ "comment::caption", "", "comment", 1, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::note", "", "comment", 2, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::place", "", "comment", 3, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::time", "", "comment", 4, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::categories", "", "comment", 5, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::rating", "", "comment", 6, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::caption", "", "comment", 1, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::note", "", "comment", 2, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::place", "", "comment", 3, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::time", "", "comment", 4, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::categories", "", "comment", 5, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::rating", "", "comment", 6, NULL, GTH_METADATA_ALLOW_NOWHERE },
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
@@ -80,23 +82,40 @@ comments__read_metadata_ready_cb (GthFileData *file_data,
 {
 	gboolean       write_comment = FALSE;
 	GthMetadata   *metadata;
+	GthStringList *comment_categories;
+	GList         *scan;
 	const char    *text;
 	GthComment    *comment;
-	GPtrArray     *keywords;
-	int            i;
 	GthStringList *categories;
 
+	if (! eel_gconf_get_boolean (PREF_STORE_METADATA_IN_FILES, TRUE)) {
+		/* if PREF_STORE_METADATA_IN_FILES is false, avoid to
+		 * synchronize the .comment metadata because the embedded
+		 * metadata is likely to be out-of-date.
+		 * Give priority to the .comment metadata which, if present,
+		 * is the most up-to-date. */
+
+		gth_comment_update_general_attributes (file_data);
+
+		return;
+	}
+
+	if (! eel_gconf_get_boolean (PREF_COMMENTS_SYNCHRONIZE, TRUE))
+		return;
+
 	comment = gth_comment_new ();
 	gth_comment_set_note (comment, g_file_info_get_attribute_string (file_data->info, "comment::note"));
+	gth_comment_set_caption (comment, g_file_info_get_attribute_string (file_data->info, "comment::caption"));
 	gth_comment_set_place (comment, g_file_info_get_attribute_string (file_data->info, "comment::place"));
 
 	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "comment::time");
 	if (metadata != NULL)
 		gth_comment_set_time_from_exif_format (comment, gth_metadata_get_raw (metadata));
 
-	keywords = gth_comment_get_categories (comment);
-	for (i = 0; i < keywords->len; i++)
-		gth_comment_add_category (comment, g_ptr_array_index (keywords, i));
+	comment_categories = (GthStringList *) g_file_info_get_attribute_object (file_data->info, "comment::categories");
+	if (comment_categories != NULL)
+		for (scan = gth_string_list_get_list (comment_categories); scan; scan = scan->next)
+			gth_comment_add_category (comment, (char *) scan->data);
 
 	/* sync embedded data and .comment data if required */
 
@@ -109,6 +128,15 @@ comments__read_metadata_ready_cb (GthFileData *file_data,
 		}
 	}
 
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::title");
+	if (metadata != NULL) {
+		text = g_file_info_get_attribute_string (file_data->info, "comment::caption");
+		if (g_strcmp0 (gth_metadata_get_raw (metadata), text) != 0) {
+			gth_comment_set_caption (comment, gth_metadata_get_raw (metadata));
+			write_comment = TRUE;
+		}
+	}
+
 	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::location");
 	if (metadata != NULL) {
 		text = g_file_info_get_attribute_string (file_data->info, "comment::place");
@@ -132,16 +160,12 @@ comments__read_metadata_ready_cb (GthFileData *file_data,
 
 	categories = (GthStringList *) g_file_info_get_attribute_object (file_data->info, "general::tags");
 	if (categories != NULL) {
-		GthStringList *comment_categories;
-
 		comment_categories = (GthStringList *) g_file_info_get_attribute_object (file_data->info, "comment::categories");
 		if (! gth_string_list_equal (categories, comment_categories)) {
-			GList *list;
 			GList *scan;
 
 			gth_comment_clear_categories (comment);
-			list = gth_string_list_get_list (categories);
-			for (scan = list; scan; scan = scan->next)
+			for (scan = gth_string_list_get_list (categories); scan; scan = scan->next)
 				gth_comment_add_category (comment, scan->data);
 			write_comment = TRUE;
 		}
@@ -211,11 +235,12 @@ gthumb_extension_deactivate (void)
 G_MODULE_EXPORT gboolean
 gthumb_extension_is_configurable (void)
 {
-	return FALSE;
+	return TRUE;
 }
 
 
 G_MODULE_EXPORT void
 gthumb_extension_configure (GtkWindow *parent)
 {
+	dlg_comments_preferences (parent);
 }
