@@ -22,6 +22,8 @@
 
 #include <config.h>
 #include <glib.h>
+#define GDK_PIXBUF_ENABLE_BACKEND
+#include <gdk-pixbuf/gdk-pixbuf-io.h>
 #include <exiv2/basicio.hpp>
 #include <exiv2/error.hpp>
 #include <exiv2/image.hpp>
@@ -186,13 +188,14 @@ attribute_is_date (const char *key)
 }
 
 
-inline static void
+static void
 set_file_info (GFileInfo  *info,
 	       const char *key,
 	       const char *description,
 	       const char *formatted_value,
 	       const char *raw_value,
-	       const char *category)
+	       const char *category,
+	       const char *type_name)
 {
 	char            *attribute;
 	GthMetadataInfo *metadata_info;
@@ -232,6 +235,7 @@ g_print ("%s (%s): %s (%s)\n", key, description, formatted_value, raw_value);
 		GthMetadataInfo info;
 
 		info.id = attribute;
+		info.type = (type_name != NULL) ? g_strdup (type_name) : NULL;
 		info.display_name = description_utf8;
 		info.category = category;
 		info.sort_order = 500;
@@ -281,7 +285,7 @@ set_attribute_from_tagset (GFileInfo  *info,
 		      "formatted", &formatted_value,
 		      "raw", &raw_value,
 		      NULL);
-	set_file_info (info, attribute, description, formatted_value, raw_value, NULL);
+	set_file_info (info, attribute, description, formatted_value, raw_value, NULL, NULL);
 }
 
 
@@ -378,7 +382,8 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 				       short_name.str().c_str(),
 				       value.str().c_str(),
 				       md->toString().c_str(),
-				       get_exif_default_category (*md));
+				       get_exif_default_category (*md),
+				       md->typeName());
 		}
 	}
 
@@ -397,7 +402,8 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 				       short_name.str().c_str(),
 				       value.str().c_str(),
 				       md->toString().c_str(),
-				       "Iptc");
+				       "Iptc",
+				       md->typeName());
 		}
 	}
 
@@ -416,7 +422,8 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 				       short_name.str().c_str(),
 				       value.str().c_str(),
 				       md->toString().c_str(),
-				       "Xmp::Embedded");
+				       "Xmp::Embedded",
+				       md->typeName());
 		}
 	}
 
@@ -531,7 +538,8 @@ exiv2_read_sidecar (GFile     *file,
 					       short_name.str().c_str(),
 					       value.str().c_str(),
 					       md->toString().c_str(),
-					       "Xmp::Sidecar");
+					       "Xmp::Sidecar",
+					       md->typeName());
 			}
 		}
 		Exiv2::XmpParser::terminate();
@@ -591,6 +599,23 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 		key = exiv2_key_from_attribute (attributes[i]);
 
 		try {
+			/* If the metadatum has no value yet, a new empty value
+			 * is created. The type is taken from Exiv2's tag
+			 * lookup tables. If the tag is not found in the table,
+			 * the type defaults to Ascii.
+			 * We always create the metadatum explicilty if the
+			 * type is available to avoid type errors.
+			 * See bug #610389 for more details.  The original
+			 * expanation is here:
+			 * http://uk.groups.yahoo.com/group/exiv2/message/1472
+			 */
+
+			GthMetadataInfo *metadatum_info = gth_main_get_metadata_info (attributes[i]);
+			if ((metadatum_info != NULL) && (metadatum_info->type != NULL)) {
+				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (metadatum_info->type));
+				Exiv2::ExifKey exif_key(key);
+				ed.add (exif_key, value.get());
+			}
 			ed[key] = gth_metadata_get_raw (metadatum);
 		}
 		catch (Exiv2::AnyError& e) {
@@ -630,6 +655,8 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 		height = gdk_pixbuf_get_height (pixbuf);
 		if (height > 0)
 			ed["Exif.Photo.PixelYDimension"] = height;
+
+		ed["Exif.Image.Orientation"] = 1;
 	}
 
 	// Update the thumbnail
@@ -676,6 +703,13 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 		char *key = exiv2_key_from_attribute (attributes[i]);
 
 		try {
+			/* See the exif data code above for an explanation. */
+			GthMetadataInfo *metadatum_info = gth_main_get_metadata_info (attributes[i]);
+			if ((metadatum_info != NULL) && (metadatum_info->type != NULL)) {
+				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (metadatum_info->type));
+				Exiv2::IptcKey iptc_key(key);
+				id.add (iptc_key, value.get());
+			}
 			id[key] = gth_metadata_get_raw (metadatum);
 		}
 		catch (Exiv2::AnyError& e) {
@@ -705,8 +739,16 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 		try {
 			const char *value = gth_metadata_get_raw (metadatum);
 
-			if ((value != NULL) && strcmp (value, "") != 0)
+			if ((value != NULL) && strcmp (value, "") != 0) {
+				/* See the exif data code above for an explanation. */
+				GthMetadataInfo *metadatum_info = gth_main_get_metadata_info (attributes[i]);
+				if ((metadatum_info != NULL) && (metadatum_info->type != NULL)) {
+					Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (metadatum_info->type));
+					Exiv2::XmpKey xmp_key(key);
+					xd.add (xmp_key, value.get());
+				}
 				xd[key] = gth_metadata_get_raw (metadatum);
+			}
 		}
 		catch (Exiv2::AnyError& e) {
 			/* we don't care about invalid key errors */
@@ -752,6 +794,7 @@ exiv2_write_metadata (SavePixbufData *data)
 		catch (Exiv2::AnyError& e) {
 			if (data->error != NULL)
 				*data->error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, e.what());
+			g_warning ("%s\n", e.what());
 			return FALSE;
 		}
 	}
@@ -786,3 +829,88 @@ exiv2_write_metadata_to_buffer (void      **buffer,
 
 	return TRUE;
 }
+
+
+#define MAX_RATIO_ERROR_TOLERANCE 0.01
+
+
+GdkPixbuf *
+exiv2_generate_thumbnail (const char *uri,
+			  const char *mime_type,
+			  int         requested_size)
+{
+	GdkPixbuf *pixbuf = NULL;
+
+	if (! _g_content_type_is_a (mime_type, "image/jpeg")
+	    && ! _g_content_type_is_a (mime_type, "image/tiff"))
+	{
+		return NULL;
+	}
+
+	try {
+		char *path;
+
+		path = g_filename_from_uri (uri, NULL, NULL);
+		if (path != NULL) {
+			Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open (path);
+			image->readMetadata ();
+			Exiv2::ExifThumbC exifThumb (image->exifData ());
+			Exiv2::DataBuf thumb = exifThumb.copy ();
+
+			if (thumb.pData_ != NULL) {
+				Exiv2::ExifData &ed = image->exifData();
+
+				long orientation = ed["Exif.Image.Orientation"].toLong();
+				long image_width = ed["Exif.Photo.PixelXDimension"].toLong();
+				long image_height = ed["Exif.Photo.PixelYDimension"].toLong();
+
+				if ((orientation == 1) && (image_width > 0) && (image_height > 0)) {
+					GInputStream *stream = g_memory_input_stream_new_from_data (thumb.pData_, thumb.size_, NULL);
+					pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+
+					if (pixbuf != NULL) {
+						/* Heuristic to find out-of-date thumbnails: the thumbnail and image aspect ratios must be equal */
+
+						double image_ratio = (((double) image_width) / image_height);
+						double thumbnail_ratio = (((double) gdk_pixbuf_get_width (pixbuf)) / gdk_pixbuf_get_height (pixbuf));
+						double ratio_delta = (image_ratio > thumbnail_ratio) ? (image_ratio - thumbnail_ratio) : (thumbnail_ratio - image_ratio);
+
+						if (ratio_delta > MAX_RATIO_ERROR_TOLERANCE) {
+							g_object_unref (pixbuf);
+							pixbuf = NULL;
+						}
+						else {
+							/* Save the original image size in the pixbuf options */
+
+							char *s = g_strdup_printf ("%ld", image_width);
+							gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Width", s);
+							g_object_set_data (G_OBJECT (pixbuf), "gnome-original-width", GINT_TO_POINTER ((int) image_width));
+							g_free (s);
+
+							s = g_strdup_printf ("%ld", image_height);
+							gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Height", s);
+							g_object_set_data (G_OBJECT (pixbuf), "gnome-original-height", GINT_TO_POINTER ((int) image_height));
+							g_free (s);
+
+							/* Set the orientation option to correctly rotate the thumbnail
+							 * in gnome_desktop_thumbnail_factory_generate_thumbnail() */
+
+							long  orientation = ed["Exif.Image.Orientation"].toLong();
+							char *orientation_s = g_strdup_printf ("%ld", orientation);
+							gdk_pixbuf_set_option (pixbuf, "orientation", orientation_s);
+							g_free (orientation_s);
+						}
+					}
+
+					g_object_unref (stream);
+				}
+			}
+
+			g_free (path);
+		}
+	}
+	catch (Exiv2::AnyError& e) {
+	}
+
+	return pixbuf;
+}
