@@ -112,6 +112,8 @@ post_photos_ready_cb (GObject      *source_object,
 	gth_task_dialog (GTH_TASK (data->conn), TRUE);
 
 	if (! picasa_web_service_post_photos_finish (picasaweb, result, &error)) {
+		if (data->conn != NULL)
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not upload the files"), &error);
 		return;
 	}
@@ -271,6 +273,8 @@ list_albums_ready_cb (GObject      *source_object,
 	_g_object_list_unref (data->albums);
 	data->albums = picasa_web_service_list_albums_finish (picasaweb, result, &error);
 	if (error != NULL) {
+		if (data->conn != NULL)
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the album list"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -327,6 +331,8 @@ connection_ready_cb (GObject      *source_object,
 			account_properties_dialog (data, data->email);
 		}
 		else {
+			if (data->conn != NULL)
+				gth_task_dialog (GTH_TASK (data->conn), TRUE);
 			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
 			gtk_widget_destroy (data->dialog);
 		}
@@ -339,7 +345,7 @@ connection_ready_cb (GObject      *source_object,
 #ifdef HAVE_GNOME_KEYRING
 	if (gnome_keyring_is_available ()) {
 		gnome_keyring_store_password (GNOME_KEYRING_NETWORK_PASSWORD,
-					      GNOME_KEYRING_SESSION,
+					      NULL,
 					      _("Picasa Web Album"),
 					      data->password,
 					      store_password_done_cb,
@@ -581,6 +587,8 @@ create_album_ready_cb (GObject      *source_object,
 
 	album = picasa_web_service_create_album_finish (picasaweb, result, &error);
 	if (error != NULL) {
+		if (data->conn != NULL)
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not create the album"), &error);
 		return;
 	}
@@ -634,7 +642,7 @@ add_album_button_clicked_cb (GtkButton *button,
 	DialogData *data = user_data;
 	GtkWidget  *dialog;
 
-	dialog = picasa_album_properties_dialog_new (g_file_info_get_display_name (data->location->info),
+	dialog = picasa_album_properties_dialog_new (g_file_info_get_edit_name (data->location->info),
 						     NULL,
 						     PICASA_WEB_ACCESS_PUBLIC);
 	g_signal_connect (dialog,
@@ -653,6 +661,8 @@ static void
 auto_select_account (DialogData *data)
 {
 	gtk_widget_hide (data->dialog);
+	if (data->conn != NULL)
+		gth_task_dialog (GTH_TASK (data->conn), FALSE);
 
 	if (data->accounts != NULL) {
 		if (data->email != NULL) {
@@ -665,6 +675,7 @@ auto_select_account (DialogData *data)
 		else {
 			GtkWidget *dialog;
 
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
 			dialog = picasa_account_chooser_dialog_new (data->accounts, data->email);
 			g_signal_connect (dialog,
 					  "response",
@@ -831,19 +842,18 @@ dlg_export_to_picasaweb (GthBrowser *browser,
 		    || g_content_type_equals (mime_type, "image/jpeg")
 		    || g_content_type_equals (mime_type, "image/png"))
 		{
-			GthFileData *new_file_data;
-
-			new_file_data = gth_file_data_dup (file_data);
-			new_file_data->thumb_loaded = FALSE;
-			total_size += g_file_info_get_size (new_file_data->info);
+			total_size += g_file_info_get_size (file_data->info);
 			n_total++;
-			data->file_list = g_list_prepend (data->file_list, new_file_data);
+			data->file_list = g_list_prepend (data->file_list, g_object_ref (file_data));
 		}
 	}
 
 	if (data->file_list == NULL) {
 		GError *error;
 
+		if (data->conn != NULL)
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+
 		error = g_error_new_literal (GTH_ERROR, GTH_ERROR_GENERIC, _("No valid file selected."));
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not export the files"), &error);
 		export_dialog_destroy_cb (NULL, data);
