@@ -294,6 +294,8 @@ list_albums_ready_cb (GObject      *source_object,
 	_g_object_list_unref (data->albums);
 	data->albums = picasa_web_service_list_albums_finish (picasaweb, result, &error);
 	if (error != NULL) {
+		if (data->conn != NULL)
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the album list"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -351,6 +353,8 @@ connection_ready_cb (GObject      *source_object,
 			account_properties_dialog (data, data->email);
 		}
 		else {
+			if (data->conn != NULL)
+				gth_task_dialog (GTH_TASK (data->conn), TRUE);
 			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
 			gtk_widget_destroy (data->dialog);
 		}
@@ -363,7 +367,7 @@ connection_ready_cb (GObject      *source_object,
 #ifdef HAVE_GNOME_KEYRING
 	if (gnome_keyring_is_available ()) {
 		gnome_keyring_store_password (GNOME_KEYRING_NETWORK_PASSWORD,
-					      GNOME_KEYRING_SESSION,
+					      NULL,
 					      _("Picasa Web Album"),
 					      data->password,
 					      store_password_done_cb,
@@ -750,6 +754,8 @@ list_photos_ready_cb (GObject      *source_object,
 	_g_object_list_unref (data->photos);
 	data->photos = picasa_web_service_list_albums_finish (picasaweb, result, &error);
 	if (error != NULL) {
+		if (data->conn != NULL)
+			gth_task_dialog (GTH_TASK (data->conn), TRUE);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the photo list"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -839,9 +845,17 @@ picasa_web_thumbnail_loader (GthFileData  *file_data,
 
 			stream = g_memory_input_stream_new_from_data (buffer, size, g_free);
 			pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, error);
-			if (pixbuf != NULL)
-				animation = gdk_pixbuf_non_anim_new (pixbuf);
+			if (pixbuf != NULL) {
+				GdkPixbuf *rotated;
 
+				rotated = gdk_pixbuf_apply_embedded_orientation (pixbuf);
+				g_object_unref (pixbuf);
+				pixbuf = rotated;
+
+				animation = gdk_pixbuf_non_anim_new (pixbuf);
+			}
+
+			g_object_unref (pixbuf);
 			g_object_unref (stream);
 		}
 
@@ -982,6 +996,8 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 			  G_CALLBACK (preferences_button_clicked_cb),
 			  data);
 
+	update_selection_status (data);
+
 	data->accounts = picasa_web_accounts_load_from_file (&data->email);
 	auto_select_account (data);
 }
