@@ -440,6 +440,8 @@ dlg_rename_series (GthBrowser *browser,
 	gtk_widget_show (data->list_view);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("preview_scrolledwindow")), data->list_view);
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("preview_label")), data->list_view);
+
 	if (data->file_list->next == NULL) {
 		GthFileData *file_data = data->file_list->data;
 		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("template_entry")), g_file_info_get_attribute_string (file_data->info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME));
@@ -476,6 +478,7 @@ dlg_rename_series (GthBrowser *browser,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (data->sort_combobox), 0);
 	gtk_widget_show (data->sort_combobox);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("sort_by_box")), data->sort_combobox);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("sort_by_label")), data->sort_combobox);
 
 	/* change case */
 
@@ -486,6 +489,7 @@ dlg_rename_series (GthBrowser *browser,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (data->change_case_combobox), 0);
 	gtk_widget_show (data->change_case_combobox);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("change_case_box")), data->change_case_combobox);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("change_case_label")), data->change_case_combobox);
 
 	dlg_rename_series_update_preview (data);
 
