@@ -1189,6 +1189,7 @@ gth_main_activate_extensions (void)
 	const char *mandatory_extensions[] = {	"file_viewer",
 						NULL };
 	const char *default_extensions[] = {	"bookmarks",
+						"burn_disc",
 						"catalogs",
 						"change_date",
 						"comments",
@@ -1198,6 +1199,7 @@ gth_main_activate_extensions (void)
 						"exiv2_tools",
 						"file_manager",
 						"file_tools",
+						"flicker",
 						"gstreamer_tools",
 						"gstreamer_utils",
 						"image_print",
